#include "WebKitPrefix.h"
#include "TSFEventRouter.h"
#include "TSFTextStore.h"

using namespace WebCore;

TSFEventRouter::TSFEventRouter(void)
:m_hWnd(NULL),m_frame(NULL)
{
	

}

TSFEventRouter::~TSFEventRouter(void)
{
	Finalize();
}

void TSFEventRouter::SetFrame(RefPtr<WebCore::Frame> frame, HWND hWnd)
{
	HWND_TSF_INFO* pInfo = FindInfo(hWnd);
	if (pInfo) {
		if(pInfo->pTextStore){
			pInfo->pTextStore->SetFrame(frame);
		}
	}
}

void TSFEventRouter::SetManager() {
	
	
	if(FAILED(CoCreateInstance(CLSID_TF_ThreadMgr,NULL,CLSCTX_INPROC_SERVER, 
			IID_ITfThreadMgr,(void**)&m_pThreadMgr)))
	{

		return ;
	}

	if(FAILED(m_pThreadMgr->Activate(&m_tfClientId)))
	{

		return ;
	}

	if(m_pThreadMgr->QueryInterface(IID_ITfKeystrokeMgr,(void**)&m_pKeystoreMgr) != S_OK)
	{
		return ;
	}

	m_textStore = new TSFTextStore();
	
	if(!m_textStore)
	{
		return;
	}
		
	m_textStore->m_hWnd = m_hWnd;

	if(FAILED(m_pThreadMgr->CreateDocumentMgr(&m_document_manger))) {
		return;
	}

	if(FAILED(m_document_manger->CreateContext(m_tfClientId,0,static_cast<ITextStoreACP*>(m_textStore),&m_context,&m_ecTextStore))) {
		return;
	}

	m_textStore->m_EditCookie = m_ecTextStore;

	if(FAILED(m_document_manger->Push(m_context))){
		return;

	}

	ITfDocumentMgr *pDocumentMgrPrev;
	m_pThreadMgr->AssociateFocus(m_hWnd, m_document_manger, &pDocumentMgrPrev);
	if (pDocumentMgrPrev){
        pDocumentMgrPrev->Release();
	}

	ITfSource *pSource = NULL;
	m_context->QueryInterface(IID_ITfSource, (void**)&pSource);
	pSource->AdviseSink(IID_ITfTextEditSink, (ITfTextEditSink*)m_textStore, &m_dwTextEditSinkCookie);
}


void TSFEventRouter::ReleaseManger() {
	
	m_pThreadMgr->Deactivate();

	if (m_context && m_dwTextEditSinkCookie) {
		ITfSource* pSource = NULL;
		m_context->QueryInterface(IID_ITfSource, (void**)&pSource);
		pSource->UnadviseSink(m_dwTextEditSinkCookie);
	}

	if(m_document_manger) 
	{
		m_document_manger->Pop(TF_POPF_ALL);
	}

	if(m_context)
	{
		m_context->Release();
		m_context = NULL;
	}

	if(m_document_manger)
	{
		m_document_manger->Release();
		m_document_manger = NULL;
	}
	

	if(m_textStore)
	{
		m_textStore->Release();
		m_textStore = NULL;
	}

	if (m_pThreadMgr)
    {
        m_pThreadMgr->Release();
    }
    CoUninitialize();
}

BOOL TSFEventRouter::InsertAtSelection(LPCWSTR psz, HWND hWnd)
{
	HWND_TSF_INFO* pInfo = FindInfo(hWnd);
	if (pInfo) {
		if(pInfo->pTextStore) {
			pInfo->pTextStore->OnTextChange(psz);
		}
	}
	SelectionChange(hWnd);

	return TRUE;
}

BOOL TSFEventRouter::SelectionChange(HWND hWnd) {

	HWND_TSF_INFO* pInfo = FindInfo(hWnd);
	if (pInfo) {
		if(pInfo->pTextStore) {

			pInfo->pTextStore->OnSelectionChange();
		}
	}
	return TRUE;
}

VOID TSFEventRouter::Initialize()
{
	CoInitialize(0);
	if(FAILED(CoCreateInstance(CLSID_TF_ThreadMgr,NULL,CLSCTX_INPROC_SERVER, 
		IID_ITfThreadMgr,(void**)&m_pThreadMgr)))
	{

		return ;
	}

	if(FAILED(m_pThreadMgr->Activate(&m_tfClientId)))
	{

		return ;
	}

	if(m_pThreadMgr->QueryInterface(IID_ITfKeystrokeMgr,(void**)&m_pKeystoreMgr) != S_OK)
	{
		return ;
	}
}

VOID TSFEventRouter::Finalize()
{
	for (int i = 0; i < m_HwndTSFInfoVec.size(); i++) {
		HWND_TSF_INFO* pInfo = m_HwndTSFInfoVec.at(i);
		if (pInfo) {
			if (pInfo->pContext && pInfo->dwTextEditSinkCookie) {
				ITfSource* pSource = NULL;
				pInfo->pContext->QueryInterface(IID_ITfSource, (void**)&pSource);
				pSource->UnadviseSink(pInfo->dwTextEditSinkCookie);
			}

			if (pInfo->pDocumentMgr) {
				pInfo->pDocumentMgr->Pop(TF_POPF_ALL);
			}
			if (pInfo->pContext) {
				pInfo->pContext->Release();
				pInfo->pContext = NULL;
			}
			if (pInfo->pPrevDocmentMgr) {
				m_pThreadMgr->AssociateFocus(pInfo->hWnd, pInfo->pPrevDocmentMgr, NULL);
			}
			if (pInfo->pDocumentMgr) {
				pInfo->pDocumentMgr->Release();
				pInfo->pDocumentMgr = NULL;
			}
			if (pInfo->pTextStore) {
				pInfo->pTextStore->Release();
			}
		}
	}
	m_HwndTSFInfoVec.clear();
	if (m_pThreadMgr) {
		m_pThreadMgr->Deactivate();
		m_pThreadMgr->Release();
	}
	CoUninitialize();
}

VOID TSFEventRouter::AddWindow( HWND hWnd )
{
	if (!m_pThreadMgr) {
		return;
	}
	ITfDocumentMgr *pDocumentMgr = NULL;
	m_pThreadMgr->CreateDocumentMgr(&pDocumentMgr);
	TSFTextStore* pTSFTextStore = new TSFTextStore();
	pTSFTextStore->m_hWnd = hWnd;
	TfEditCookie ecTextStore;
	ITfContext* pContext = NULL;
	if (!pDocumentMgr) {
		return;
	}
	pDocumentMgr->CreateContext(m_tfClientId, 0, (ITextStoreACP*)pTSFTextStore, &pContext, &ecTextStore);
	pTSFTextStore->m_EditCookie = ecTextStore;
	pDocumentMgr->Push(pContext);
	ITfDocumentMgr* pPrevDocumentMgr = NULL;
	//m_pThreadMgr->AssociateFocus(hWnd, pDocumentMgr, &pPrevDocumentMgr);
	ITfSource *pSource = NULL;
	if (!pContext) {
		return;
	}
	pContext->QueryInterface(IID_ITfSource, (void**)&pSource);
	DWORD dwTextEditSinkCookie = 0;
	if (!pSource) {
		return;
	}
	pSource->AdviseSink(IID_ITfTextEditSink, (ITfTextEditSink*)pTSFTextStore, &dwTextEditSinkCookie);

	HWND_TSF_INFO *pInfo = new HWND_TSF_INFO();
	pInfo->hWnd = hWnd;
	pInfo->pDocumentMgr = pDocumentMgr;
	pInfo->pContext = pContext;
	pInfo->pTextStore = pTSFTextStore;
	pInfo->pPrevDocmentMgr = pPrevDocumentMgr;
	pInfo->dwTextEditSinkCookie = dwTextEditSinkCookie;

	m_HwndTSFInfoVec.append(pInfo);

}

VOID TSFEventRouter::DeleteWindow( HWND hWnd )
{
	for(int i = 0; i < m_HwndTSFInfoVec.size(); i++) {
		HWND_TSF_INFO* pInfo = m_HwndTSFInfoVec.at(i);
		if (pInfo) {
			if (pInfo->hWnd == hWnd) {
				if (pInfo->pContext && pInfo->dwTextEditSinkCookie) {
					ITfSource* pSource = NULL;
					pInfo->pContext->QueryInterface(IID_ITfSource, (void**)&pSource);
					pSource->UnadviseSink(pInfo->dwTextEditSinkCookie);
				}

				if (pInfo->pDocumentMgr) {
					pInfo->pDocumentMgr->Pop(TF_POPF_ALL);
				}
				if (pInfo->pContext) {
					pInfo->pContext->Release();
					pInfo->pContext = NULL;
				}
				if (pInfo->pPrevDocmentMgr) {
					m_pThreadMgr->AssociateFocus(pInfo->hWnd, pInfo->pPrevDocmentMgr, NULL);
				}
				if (pInfo->pDocumentMgr) {
					pInfo->pDocumentMgr->Release();
					pInfo->pDocumentMgr = NULL;
				}
				if (pInfo->pTextStore) {
					pInfo->pTextStore->Release();
				}

				m_HwndTSFInfoVec.remove(i);

				return;
			}
		}
	}
}

TSFEventRouter::HWND_TSF_INFO* TSFEventRouter::FindInfo( HWND hWnd )
{
	HWND_TSF_INFO* pInfo = NULL;
	for (int i = 0; i < m_HwndTSFInfoVec.size(); i++) {
		HWND_TSF_INFO* pInfoTemp = m_HwndTSFInfoVec.at(i);
		if (pInfoTemp) {
			if (pInfoTemp->hWnd == hWnd) {
				pInfo = pInfoTemp;
				break;
			}
		}
	}
	return pInfo;
}

