#include "WebKitPrefix.h"
#include "TSFTextStore.h"
#include "initguid.h"
#include "InputScope.h"
#include "ExceptionCode.h"
#include "WebCore/NotImplemented.h"
#include <msctf.h>
#include <WebCore/FrameView.h>
#include <WebCore/Page.h>
#include <WebCore/FocusController.h>
#include "tsattrs.h"
#include <strsafe.h>
#include "WebCoreInstanceHandle.h"

using namespace std;
using namespace WebCore;
using WTF::String;
using WebCore::Frame;

const TsViewCookie kViewCookie = 0;
class Frame;

bool g_bool = false;
VOID MyDbgLog(LPCTSTR lpszFmt, ...)
{
	TCHAR szBuff2[1024] = TEXT("");
	va_list args;
	va_start(args, lpszFmt);
	StringCchVPrintf(szBuff2, 1024, lpszFmt, args);
	va_end(args);

	TCHAR szBuff[1024] = TEXT("");
	static int nCnt = 0;
	StringCchPrintf(szBuff, 1024, TEXT("[%04d]: %s"), nCnt++, szBuff2);
	OutputDebugString(szBuff);
}
#define DBGOUT MyDbgLog


const GUID *c_rgSupportedAttr[7] = {&GUID_PROP_INPUTSCOPE,
                                    &TSATTRID_Font_Style_Height,
                                    &TSATTRID_Font_FaceName,
                                    &TSATTRID_Font_SizePts,
                                    &TSATTRID_Text_ReadOnly,
                                    &TSATTRID_Text_Orientation,
                                    &TSATTRID_Text_VerticalWriting};

TSFTextStore::TSFTextStore()
:m_frame(NULL),m_hWnd(NULL)
{
	_pCurrentCompositionView = NULL;
	_cRef = 1;
	m_dwCurLock = 0;

	m_uSelStart = 0;
	m_uSelEnd = 0;

	CoCreateInstance(CLSID_TF_DisplayAttributeMgr,
		NULL, CLSCTX_INPROC_SERVER, 
		IID_ITfDisplayAttributeMgr, 
		(void**)&m_pdam);

	CoCreateInstance(CLSID_TF_CategoryMgr,
		NULL, CLSCTX_INPROC_SERVER,
		IID_ITfCategoryMgr,
		(void**)&m_pcat);
}

TSFTextStore::~TSFTextStore(void)
{
	if(m_pdam) {
		ULONG uRef = m_pdam->Release();
		if (uRef == 0) {
			m_pdam = NULL;
		}
	}
	if (m_pcat) {
		ULONG uRef = m_pcat->Release();
		if (uRef == 0) {
			m_pcat = NULL;
		}
	}
}


void TSFTextStore::SetFrame(RefPtr<WebCore::Frame> frame)
{
	m_frame = frame;
}

void TSFTextStore::OnTextChange(LPCWSTR psz)
{
	LONG acpStart = 0;
	LONG acpEnd = 0;

	//GetRange(acpStart,acpEnd);
	acpStart = m_uSelStart;
	acpEnd = m_uSelEnd;

	LONG acpNewEnd = 0 ;
	acpStart += lstrlen(psz);
	acpNewEnd = acpStart;

    if (m_pSink)
    {
        TS_TEXTCHANGE textChange;
        textChange.acpStart = acpStart;
        textChange.acpOldEnd = acpEnd;
        textChange.acpNewEnd = acpNewEnd;
        m_pSink->OnTextChange(0, &textChange);
    }

}

RefPtr<Range> TSFTextStore::GetRange(LONG &acpStart,LONG &acpEnd)
{
	RefPtr<Range> range = m_frame.get()->editor()->hasComposition() ? m_frame.get()->editor()->compositionRange() : m_frame.get()->selection()->selection().toNormalizedRange();
	if(range == NULL){
		acpStart = 0;
		acpEnd = 0;
	} else {
		acpStart = range.get()->startOffset();
		acpEnd =  range.get()->endOffset();
	}
	return range;
}

void TSFTextStore::OnSelectionChange()
{
	if (m_pSink){
        m_pSink->OnSelectionChange();
	}
}

//IUnKnow
STDMETHODIMP TSFTextStore::QueryInterface(const IID &riid, void **ppvObj)
{
	*ppvObj = NULL;

    if (IsEqualIID(riid, IID_IUnknown) ||
        IsEqualIID(riid, IID_ITextStoreACP))
    {
        *ppvObj = (ITextStoreACP *)this;
    }
    else if (IsEqualIID(riid, IID_ITfContextOwnerCompositionSink))
    {
        *ppvObj = (ITfContextOwnerCompositionSink *)this;
    }
	else if (IsEqualIID(riid, IID_ITfTextEditSink))
	{
		*ppvObj = (ITfTextEditSink*)this;
	}

    if (*ppvObj)
    {
        AddRef();
        return S_OK;
    }
    return E_NOINTERFACE;

}

STDMETHODIMP_(ULONG) TSFTextStore::AddRef()
{
	return ++_cRef;
}

STDMETHODIMP_(ULONG) TSFTextStore::Release()
{
    long cr;

    cr = --_cRef;

    if (cr == 0)
    {
        delete this;
    }

    return cr;
}

BOOL TSFTextStore::GetCompostionStatus(ITfContext* context, TfEditCookie read_only_edit_cookie)
{
	BOOL bRet = FALSE;
	//Display Attribute And Composition State
	const GUID* guidp[] = {&GUID_PROP_ATTRIBUTE, &GUID_PROP_COMPOSING};

	ITfReadOnlyProperty *pProperty = NULL;
	if (SUCCEEDED(context->TrackProperties(guidp, _countof(guidp), NULL, 0, &pProperty))) {
		// We read the display attribute for entire range.
		// It could be optimized by filtering the only delta with ITfEditRecord interface. 
		ITfRange *pRangeEntire = NULL;
		ITfRange *pRangeEnd = NULL;
		lines.clear();
		if (SUCCEEDED(context->GetStart(read_only_edit_cookie, &pRangeEntire)) &&
			SUCCEEDED(context->GetEnd(read_only_edit_cookie, &pRangeEnd)) &&
			SUCCEEDED(pRangeEntire->ShiftEndToRange(read_only_edit_cookie, pRangeEnd, TF_ANCHOR_END)))
		{
			IEnumTfRanges *pEnumRanges;
			if (SUCCEEDED(pProperty->EnumRanges(read_only_edit_cookie, &pEnumRanges, pRangeEntire)))
			{
				ITfRange* pRange = NULL;
				VARIANT var;
				VariantInit(&var);
				while (pEnumRanges->Next(1, &pRange, NULL) == S_OK) {
					WCHAR szBuf[100] = TEXT("");
					ULONG cch= 0;
					pRange->GetText(read_only_edit_cookie, 0, szBuf, 100, &cch);
					BOOL bCompositon = FALSE;
					BOOL bDisplay = FALSE;
					TF_DISPLAYATTRIBUTE da;
					if (SUCCEEDED(pProperty->GetValue(read_only_edit_cookie, pRange, &var))) {
						IEnumTfPropertyValue *pEnumProp = NULL;
						if (SUCCEEDED(var.punkVal->QueryInterface(IID_IEnumTfPropertyValue, (void**)&pEnumProp))) {
							TF_PROPERTYVAL pPropVal;
							while (pEnumProp->Next(1, &pPropVal, NULL) == S_OK) {
								if (IsEqualGUID(pPropVal.guidId, GUID_PROP_COMPOSING)) {
									bCompositon = pPropVal.varValue.lVal == TRUE;
								}
								else if (IsEqualGUID(pPropVal.guidId, GUID_PROP_ATTRIBUTE)) {
									TfGuidAtom gui_atom = (TfGuidAtom)pPropVal.varValue.lVal;
									if (m_pdam)
									{
										if (m_pcat)
										{
											GUID guid;
											m_pcat->GetGUID(gui_atom, &guid);
											ITfDisplayAttributeInfo *pDAI;
											if ((m_pdam != NULL) && SUCCEEDED(m_pdam->GetDisplayAttributeInfo(guid, &pDAI, NULL)))
											{
												pDAI->GetAttributeInfo(&da);
												pDAI->Release();
												bDisplay = TRUE;
											}
										}
									}
								}
								VariantClear(&pPropVal.varValue);
							}
						}
					}
					VariantClear(&var);

					if (bDisplay) {
						bRet = TRUE;
						ITfRangeACP *pRangeACP = NULL;
						if (SUCCEEDED(pRange->QueryInterface(IID_ITfRangeACP, (void**)&pRangeACP))) {
							LONG acpAnchor = 0, cch = 0;
							pRangeACP->GetExtent(&acpAnchor, &cch);
							if (bCompositon) {
								CompositionUnderline line;
								line.startOffset = acpAnchor;
								line.endOffset = acpAnchor + cch;
								line.thick = da.fBoldLine;
								line.color = Color(0,0,0);
								lines.append(line);
							}
						}
					}
				}
			}
			if (pRangeEntire)
			{
				pRangeEntire->Release();
			}
			if (pRangeEnd)
			{
				pRangeEnd->Release();
			}

		}
	}
	return bRet;
}


STDMETHODIMP TSFTextStore::OnEndEdit(ITfContext* context,TfEditCookie read_only_edit_cookie,ITfEditRecord* edit_record)
{
	//DBGOUT(TEXT(">>>>>>>>>>>>>>>> ")TEXT(__FUNCTION__)TEXT("\n"));
	if (!context || !edit_record)
	{
		return E_INVALIDARG;
	}
	const GUID* guidp[] = {&GUID_PROP_ATTRIBUTE, &GUID_PROP_COMPOSING};

	IEnumTfRanges *pEnumTfRanges = NULL;
	if (SUCCEEDED(edit_record->GetTextAndPropertyUpdates(TF_GTP_INCL_TEXT, guidp, 2, &pEnumTfRanges)) )
	{
		ITfRange *pRangeTemp = NULL;
		if (pEnumTfRanges->Next(1, &pRangeTemp, NULL) == S_OK)
		{
			if (_pCurrentCompositionView) {
				if (GetCompostionStatus(context, read_only_edit_cookie)) {
					m_frame->editor()->setComposition(m_stringBuffer, lines, m_uSelEnd, m_uSelEnd);
				}
			}
			else {
				m_frame->editor()->confirmComposition(m_stringBuffer);
				lines.clear();
				m_uSelStart = 0;
				m_uSelEnd = 0;
				m_stringBuffer = emptyString();
			}

		}

	}

	return S_OK;
}


STDMETHODIMP TSFTextStore::AdviseSink(REFIID riid, IUnknown *punk, DWORD dwMask)
{
	if(! IsEqualGUID(riid,IID_ITextStoreACPSink))
	{
		return E_INVALIDARG;
	}

	if(FAILED(punk->QueryInterface(IID_ITextStoreACPSink,(void **)&m_pSink)))
	{
		return E_NOINTERFACE;
	}
	return S_OK;

}

STDMETHODIMP TSFTextStore::UnadviseSink(IUnknown *punk)
{
	if(m_pSink) {
		m_pSink->Release();
		m_pSink = NULL;
	}
	return S_OK;
}

STDMETHODIMP TSFTextStore::RequestLock(DWORD dwLockFlags, HRESULT *phrSession) {


	if(!m_pSink) {
		return E_UNEXPECTED;
	}

	if(!phrSession)
	{
		return E_UNEXPECTED;
	}
	if (m_dwCurLock) {
		if (dwLockFlags & TS_LF_SYNC) {
			*phrSession = TS_E_SYNCHRONOUS;
			return S_OK;
		}
		m_LockQueue.append(dwLockFlags & TS_LF_READWRITE);
		*phrSession = TS_S_ASYNC;
		return S_OK;
	}
	m_dwCurLock = dwLockFlags & TS_LF_READWRITE;


	*phrSession =m_pSink->OnLockGranted(m_dwCurLock);

	m_dwCurLock = 0;

	while (!m_LockQueue.isEmpty()) {
		m_dwCurLock = m_LockQueue.first();
		m_LockQueue.remove(0);
		m_pSink->OnLockGranted(m_dwCurLock);
		m_dwCurLock = 0;
	}

	return S_OK;
}

STDMETHODIMP TSFTextStore::GetStatus(TS_STATUS *pdcs)
{
	pdcs->dwDynamicFlags = 0;
	pdcs->dwStaticFlags =0; //TS_SS_TRANSITORY | TS_SS_NOHIDDENTEXT;
	return S_OK;
}

STDMETHODIMP TSFTextStore::QueryInsert(LONG acpInsertStart, LONG acpInsertEnd, ULONG cch, LONG *pacpResultStart, LONG *pacpResultEnd)
{
	if (!((0 <= acpInsertStart)&&(acpInsertStart<=acpInsertEnd)&&(acpInsertEnd<=m_stringBuffer.length())))
	{
		acpInsertStart = acpInsertEnd = 0;
	}
    *pacpResultStart = acpInsertStart;
    *pacpResultEnd = acpInsertEnd;
    return S_OK;
}


STDMETHODIMP TSFTextStore::GetSelection(ULONG ulIndex, ULONG ulCount, TS_SELECTION_ACP *pSelection, ULONG *pcFetched)
{
    *pcFetched = 0;
    if ((ulCount > 0) && ((ulIndex == 0) || (ulIndex == TS_DEFAULT_SELECTION)))
    {
		//LONG acpStart = 0;
		//LONG acpEnd = 0;
		//GetRange(acpStart,acpEnd);
		//
		//pSelection[0].acpStart = acpStart;
		//pSelection[0].acpEnd = acpEnd;
		pSelection[0].style.ase = TS_AE_END;
		pSelection[0].style.fInterimChar = FALSE;

		pSelection[0].acpStart = m_uSelStart;
		pSelection[0].acpEnd = m_uSelEnd;
		*pcFetched = 1;//it needs comentout when debugging 
		
    }

	return S_OK;
}


STDMETHODIMP TSFTextStore::SetSelection(ULONG ulCount, const TS_SELECTION_ACP *pSelection)
{
	//DBGOUT(TEXT(">>>>>>>>>>>>>>>> ")TEXT(__FUNCTION__)TEXT("\n"));
	//for (ULONG u = 0;u<ulCount;u++) {
	//	DBGOUT(TEXT("pSlection[%d]: acpStart %d, acpEnd %d\n"), u, pSelection[u].acpStart, pSelection[u].acpEnd);
	//}
    if (ulCount > 0)
    { 
		//ExceptionCode ec = 0;
		//LONG startPos;
		//LONG endPos;
		//RefPtr<Range> range = GetRange(startPos,endPos);//m_frame.get()->selection()->selection().toNormalizedRange();
		//if(range != NULL){
		//	range.get()->setStart(range->startContainer(ec),pSelection[0].acpStart,ec);
		//	range.get()->setEnd(range->startContainer(ec),pSelection[0].acpEnd,ec);
		//}
		//m_frame->editor()->setSelectionOffsets(pSelection[0].acpStart, pSelection[0].acpEnd);
		//Position base = m_frame->selection()->base().downstream();
		//Node* basenode = base.deprecatedNode();
		//RefPtr<Range> selectedrange = Range::create(basenode->document(), basenode, pSelection[0].acpStart, basenode, pSelection[0].acpEnd);
		//m_frame->selection()->setSelectedRange(selectedrange.get(), DOWNSTREAM, false);
		//m_frame->selection()->setSelectedRange(range.get(), DOWNSTREAM, false);
		LONG acpStart = pSelection[0].acpStart;
		LONG acpEnd = pSelection[0].acpEnd;
		if (!((0<=acpStart)&&(acpStart<=acpEnd)&&(acpEnd<=m_stringBuffer.length())))
		{
			return TS_E_INVALIDPOS;
		}

		m_uSelStart = pSelection[0].acpStart;
		m_uSelEnd = pSelection[0].acpEnd;
		//OnSelectionChange();
    }
	//DBGOUT(TEXT(__FUNCTION__)TEXT(">>>>>>>>>>>>")TEXT("\n"));
    return S_OK;;
}

STDMETHODIMP TSFTextStore::GetText(LONG acpStart, LONG acpEnd, __out_ecount(cchPlainReq) WCHAR *pchPlain, ULONG cchPlainReq, ULONG *pcchPlainOut, TS_RUNINFO *prgRunInfo, ULONG ulRunInfoReq, ULONG *pulRunInfoOut, LONG *pacpNext)
{

	if(!pcchPlainOut || !pulRunInfoOut)
	{
		return E_INVALIDARG;
	}

	if(!pchPlain && cchPlainReq != 0)
	{
		return E_INVALIDARG;
	}

	if(!prgRunInfo && ulRunInfoReq != 0)
	{
		return E_INVALIDARG;
	}
	LONG string_buffer_size = m_stringBuffer.length();

	if (acpEnd == -1)
	{
        acpEnd = string_buffer_size;
	}
	if( !((0<= acpStart) && (acpStart <= acpEnd) && (acpEnd <= string_buffer_size)))
	{
		return TF_E_INVALIDPOS;
	}

    acpEnd = min(acpEnd, acpStart + (int)cchPlainReq);

	*pcchPlainOut = acpEnd - acpStart;


	WTF::String result = m_stringBuffer.substring(acpStart, *pcchPlainOut);
	for(size_t i=0; i < result.length(); i++) {
		pchPlain[i] = result[i];
	}

    *pcchPlainOut = acpEnd - acpStart;
    if (ulRunInfoReq)
    {
        prgRunInfo[0].uCount = acpEnd - acpStart;
        prgRunInfo[0].type = TS_RT_PLAIN;
        *pulRunInfoOut = 1;
    }

    *pacpNext = acpEnd;
    return S_OK;
}

STDMETHODIMP TSFTextStore::SetText(DWORD dwFlags, LONG acpStart, LONG acpEnd, __in_ecount(cch) const WCHAR *pchText, ULONG cch, TS_TEXTCHANGE *pChange)
{
	TS_SELECTION_ACP selection;
	selection.acpStart = acpStart;
	selection.acpEnd = acpEnd;
	selection.style.ase = TS_AE_NONE;
	selection.style.fInterimChar = 0;

	HRESULT ret;
	
	ret = SetSelection(1,&selection);
	if(ret != S_OK){
		return ret;
	}

	TS_TEXTCHANGE change;
	ret = InsertTextAtSelection(0,pchText,cch,&acpStart,&acpEnd,&change);
	if(ret != S_OK) {
		return ret;
	}

	if(pChange)
	{
		*pChange = change;
	}

	//DBGOUT(TEXT(__FUNCTION__)TEXT(">>>>>>>>>>>>")TEXT("\n"));
    return S_OK;
}

STDMETHODIMP TSFTextStore::GetFormattedText(LONG acpStart, LONG acpEnd, IDataObject **ppDataObject)
{
    return E_NOTIMPL;
}


STDMETHODIMP TSFTextStore::GetEmbedded(LONG acpPos, REFGUID rguidService, REFIID riid, IUnknown **ppunk)
{
	notImplemented();
    return E_NOTIMPL;
}

STDMETHODIMP TSFTextStore::InsertEmbedded(DWORD dwFlags, LONG acpStart, LONG acpEnd, IDataObject *pDataObject, TS_TEXTCHANGE *pChange)
{
	notImplemented();
    return E_NOTIMPL;
}



STDMETHODIMP TSFTextStore::RequestSupportedAttrs(DWORD dwFlags, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs)
{
	if(!paFilterAttrs) {
		return E_INVALIDARG;
	}

	//for (size_t i = 0; i < ARRAYSIZE(c_rgSupportedAttr); i++)
 //   {
 //       if (cFilterAttrs)
 //       {
 //           BOOL fFound = FALSE;
 //           for (ULONG j = 0; j < cFilterAttrs; j++)
 //           {
 //               if (IsEqualGUID(*c_rgSupportedAttr[i], paFilterAttrs[j]))
 //               {
	//				return S_OK;
 //               }
 //           }

 //           
 //       }
	//}
	return E_FAIL;
}


STDMETHODIMP TSFTextStore::RequestAttrsAtPosition(LONG acpPos, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs, DWORD dwFlags)
{
	if(!paFilterAttrs) {
		return E_INVALIDARG;
	}

	//for (size_t i = 0; i < ARRAYSIZE(c_rgSupportedAttr); i++)
 //   {
 //       if (cFilterAttrs)
 //       {
 //           BOOL fFound = FALSE;
 //           for (ULONG j = 0; j < cFilterAttrs; j++)
 //           {
 //               if (IsEqualGUID(*c_rgSupportedAttr[i], paFilterAttrs[j]))
 //               {
	//				return S_OK;
 //               }
 //           }
 //           
 //       }
	//}

	return E_FAIL;
}

STDMETHODIMP TSFTextStore::RequestAttrsTransitioningAtPosition(LONG acpPos, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs, DWORD dwFlags)
{
    return E_NOTIMPL;
}

STDMETHODIMP TSFTextStore::FindNextAttrTransition(LONG acpStart, LONG acpHalt, ULONG cFilterAttrs, const TS_ATTRID *paFilterAttrs, DWORD dwFlags, LONG *pacpNext, BOOL *pfFound, LONG *plFoundOffset)
{
    *pacpNext = 0;
    *pfFound = FALSE;
    *plFoundOffset = 0;
    return S_OK;
}

STDMETHODIMP TSFTextStore::RetrieveRequestedAttrs(ULONG ulCount, TS_ATTRVAL *paAttrVals, ULONG *pcFetched)
{
	if(!pcFetched) {
		return E_INVALIDARG;
	}

	if(!paAttrVals) {
		return E_INVALIDARG;
	}

	*pcFetched = 0;
	if(ulCount == 0) {
		return S_OK;
	}

	//paAttrVals[0].dwOverlapId = 0;
	//paAttrVals[0].idAttr = GUID_PROP_INPUTSCOPE;
	//paAttrVals[0].varValue.vt = VT_UNKNOWN;
	//paAttrVals[0].varValue.punkVal = NULL;
	//	
	//*pcFetched = 1;

    return S_OK;
}

STDMETHODIMP TSFTextStore::GetEndACP(LONG* acp) 
{
	if(!acp) {
		return E_INVALIDARG;
	}
	
	//readlock? is justified here in chromium	
	*acp =  m_stringBuffer.length(); 
	return S_OK;

}

STDMETHODIMP TSFTextStore::GetActiveView(TsViewCookie *pvcView)
{
    *pvcView = kViewCookie;;
    return S_OK;
}

STDMETHODIMP TSFTextStore::GetACPFromPoint(TsViewCookie vcView, const POINT *pt, DWORD dwFlags, LONG *pacp)
{
	notImplemented();
    return E_NOTIMPL;
}


STDMETHODIMP TSFTextStore::GetTextExt(TsViewCookie vcView, LONG acpStart, LONG acpEnd, RECT *prc, BOOL *pfClipped)
{
	//DBGOUT(TEXT(">>>>>>>>")TEXT(__FUNCTION__)TEXT("\n"));
	if(vcView != kViewCookie){
		return E_INVALIDARG;
	}
	//if((acpStart == 0) && (acpEnd == 0)) {
	//	return E_INVALIDARG;
	//}

	LONG selStart, selEnd;
	GetRange(selStart,selEnd);
	Position base = m_frame->selection()->base();
	Node* baseNode = base.deprecatedNode();
	if (!baseNode) {
		baseNode = m_frame.get()->page()->focusController()->focusedOrMainFrame()->document()->focusedNode();
		if (!baseNode) {
			return TS_E_INVALIDPOS;
		}
	}
	RefPtr<Range> compostioningpart = Range::create(baseNode->document(), baseNode, selStart+acpStart, baseNode, selStart+acpEnd);
	IntRect intrc = m_frame->editor()->firstRectForRange(compostioningpart.get());
	intrc = m_frame->view()->convertToRootView(intrc);
	intrc.move(-m_frame->view()->scrollOffset().width(), -m_frame->view()->scrollOffset().height());
	*prc = intrc;
	//RECT result;

	//IntRect resultRect;
	//WebCore::Frame* tempFrame = m_frame.get();
	//
	//LONG start_pos = 0;
	//LONG end_pos = 0; 

	//GetRange(start_pos,end_pos);

	//if(start_pos == end_pos) {
	//	
	//	RefPtr<Range> range = tempFrame->editor()->hasComposition() ? tempFrame->editor()->compositionRange() : tempFrame->selection()->selection().toNormalizedRange();
	//	if (range) {
	//		ExceptionCode ec = 0;
	//		IntRect caret;
	//		RefPtr<Range> tempRange = range->cloneRange(ec);
	//		tempRange->setStart(tempRange->startContainer(ec), tempRange->startOffset(ec) + start_pos, ec);
	//		resultRect = tempFrame->editor()->firstRectForRange(tempRange.get());
	//	//	resultRect = tempFrame->view()->contentsToWindow(caret);
	//		
	//		prc->left = resultRect.x();
	//		prc->top = resultRect.y();
	//		prc->right = resultRect.x() + resultRect.width();
	//		prc->bottom = resultRect.y() + resultRect.height();
	//	} else {
	//		prc->left = 0;
	//		prc->top = 0;
	//		prc->right = 0;
	//		prc->bottom = 0;
	//	}
	//} else {
	//	RECT rcClient;
	//	GetClientRect(m_hWnd, &rcClient);
	//	prc->left = rcClient.left;
	//	prc->right = rcClient.right;
	//	prc->top = rcClient.top;
	//	prc->bottom = rcClient.bottom;
	//		
	//}
	
	ClientToScreen(m_hWnd,(LPPOINT) &prc->left);
	ClientToScreen(m_hWnd,(LPPOINT) &prc->right);
	*pfClipped = FALSE;
	//DBGOUT(TEXT(__FUNCTION__)TEXT(">>>>>>>>")TEXT("\n"));
	return S_OK;	

}

STDMETHODIMP TSFTextStore::GetScreenExt(TsViewCookie vcView, RECT *prc)
{
	if(vcView != kViewCookie) {
		return E_INVALIDARG;
	}
	if(!prc) {
		return E_INVALIDARG;
	}
	
	SetRect(prc,0,0,0,0);

	if(! IsWindow(m_hWnd))
	{
		return E_FAIL;
	}
	
	RECT client_rect = {};
	
	if(!GetClientRect(m_hWnd, &client_rect))
	{
		return E_FAIL;
	}
	
	POINT left_top = {client_rect.left,client_rect.top};
	POINT right_bottom = {client_rect.right,client_rect.bottom};
	
	if(!ClientToScreen(m_hWnd, &left_top))
	{
		return E_FAIL;
	}
	if(!ClientToScreen(m_hWnd,&right_bottom))
	{
		return E_FAIL;
	}
    
	prc->left = left_top.x;
	prc->top = left_top.y;
	prc->right = right_bottom.x;
	prc->bottom = right_bottom.y;

    return S_OK;
}

STDMETHODIMP TSFTextStore::GetWnd(TsViewCookie vcView, HWND *phwnd)
{
	if(!phwnd){
		return E_INVALIDARG;
	}
	if(vcView != kViewCookie) {
		return E_INVALIDARG;
	}

	*phwnd = m_hWnd;

    return S_OK;
}

STDMETHODIMP TSFTextStore::QueryInsertEmbedded(const GUID *pguidService, const FORMATETC *pFormatEtc, BOOL *pfInsertable)
{
	if (!pFormatEtc)
		return E_INVALIDARG;
	// We don't support any embedded objects.
	if (pfInsertable)
		*pfInsertable = FALSE;
	return S_OK;
	//notImplemented();
    //return E_NOTIMPL;
}

STDMETHODIMP TSFTextStore::InsertTextAtSelection(DWORD dwFlags, __in_ecount(cch) const WCHAR *pchText, ULONG cch, LONG *pacpStart, LONG *pacpEnd, TS_TEXTCHANGE *pChange)
{
	//DBGOUT(TEXT(">>>>>>>>")TEXT(__FUNCTION__)TEXT("\n"));
	//{
	//	DBGOUT(TEXT("pchText is %s, cch: %d, acpStart: %d, acpEnd: %d\n"), pchText, cch, *pacpStart, *pacpEnd);
	//}
	LONG start_pos;
	LONG end_pos;

	start_pos = m_uSelStart;
	end_pos = m_uSelEnd;

	if(dwFlags & TS_IAS_QUERYONLY) {
		if(pacpStart) {
			*pacpStart = start_pos;
		}
		if(pacpEnd) {
			*pacpEnd = start_pos + cch;
		}
		return S_OK;
	}

	if(!pchText) {
		return E_INVALIDARG;
	}

	String compositionStr = String(pchText, cch);	
	m_stringBuffer = m_stringBuffer.substring(0, start_pos) + compositionStr + m_stringBuffer.substring(end_pos);
	//m_stringBuffer = m_stringBuffer.substring(0, *pacpStart) + compositionStr + m_stringBuffer.substring(*pacpEnd);

	/*if (cch == 0) {
		m_frame->editor()->deleteWithDirection(DirectionLeft, CharacterGranularity, false, true);
	}*/

	if(pacpStart) {
		*pacpStart = start_pos;
	}
	if(pacpEnd) {
		*pacpEnd = start_pos + cch;
	}

	if(pChange) {
		pChange->acpStart = start_pos;
		pChange->acpOldEnd = end_pos;
		pChange->acpNewEnd = start_pos + cch;
	}

	OnSelectionChange();

	return S_OK;
}

STDMETHODIMP TSFTextStore::InsertEmbeddedAtSelection(DWORD dwFlags, IDataObject *pDataObject, LONG *pacpStart, 
        LONG *pacpEnd, TS_TEXTCHANGE *pChange)
{
	notImplemented();
    return E_NOTIMPL;
}


//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------

STDAPI TSFTextStore::OnStartComposition(ITfCompositionView *pComposition, BOOL *pfOk)
{
    if (_pCurrentCompositionView)
    {
        _pCurrentCompositionView->Release();
        _pCurrentCompositionView = NULL;
    }

	_pCurrentCompositionView = pComposition;
    _pCurrentCompositionView->AddRef();

    *pfOk = TRUE;
    return S_OK;
}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------

STDAPI TSFTextStore::OnUpdateComposition(ITfCompositionView *pComposition,
                                ITfRange *pRangeNew)
{
    if (_pCurrentCompositionView)
    {
        _pCurrentCompositionView->Release();
        _pCurrentCompositionView = NULL;
    }

    _pCurrentCompositionView = pComposition;
    _pCurrentCompositionView->AddRef();
    return S_OK;
}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------

STDAPI TSFTextStore::OnEndComposition(ITfCompositionView *pComposition)
{
    if (_pCurrentCompositionView)
    {
        _pCurrentCompositionView->Release();
        _pCurrentCompositionView = NULL;
    }
    return S_OK;
}
