/*
 * Copyright (C) 2009 Apple Inc. All Rights Reserved.
 * Copyright (C) 2009 Brent Fulgham. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef PrintWebUIDelegatePrivate2_h
#define PrintWebUIDelegatePrivate2_h

#include <WebKit/WebKit.h>
#include "MYDEBUG.h"

class PrintWebUIDelegatePrivate2 : public IWebUIDelegatePrivate2 {
public:
    PrintWebUIDelegatePrivate2() : m_refCount(1) {}

    // IUnknown
    virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void** ppvObject);
    virtual ULONG STDMETHODCALLTYPE AddRef(void);
    virtual ULONG STDMETHODCALLTYPE Release(void);

    virtual HRESULT STDMETHODCALLTYPE webViewScrolled( IWebView* sender);
    virtual HRESULT STDMETHODCALLTYPE webViewAddMessageToConsole( IWebView* sender,  BSTR message,  int lineNumber,  BSTR url,  BOOL isError);
    virtual HRESULT STDMETHODCALLTYPE webViewShouldInterruptJavaScript( IWebView* sender,  BOOL* result);
    virtual HRESULT STDMETHODCALLTYPE webViewReceivedFocus( IWebView* sender);
    virtual HRESULT STDMETHODCALLTYPE webViewLostFocus( IWebView* sender,  OLE_HANDLE loseFocusToHWnd);
    virtual HRESULT STDMETHODCALLTYPE doDragDrop( IWebView* sender,  IDataObject* dataObject,  IDropSource* dropSource,  DWORD okEffect,  DWORD* performedEffect);
    virtual HRESULT STDMETHODCALLTYPE webViewGetDlgCode( IWebView* sender,  UINT keyCode,  LONG_PTR* code);
    virtual HRESULT STDMETHODCALLTYPE webViewPainted( IWebView* sender);
    virtual HRESULT STDMETHODCALLTYPE exceededDatabaseQuota( IWebView* sender,  IWebFrame* frame,  IWebSecurityOrigin* origin,  BSTR databaseIdentifier);
    virtual HRESULT STDMETHODCALLTYPE embeddedViewWithArguments( IWebView* sender,  IWebFrame* frame,  IPropertyBag* arguments,  IWebEmbeddedView** view);
    virtual HRESULT STDMETHODCALLTYPE unused3();
    virtual HRESULT STDMETHODCALLTYPE webViewClosing( IWebView* sender);
    virtual HRESULT STDMETHODCALLTYPE webViewSetCursor( IWebView* sender,  OLE_HANDLE cursor);
    virtual HRESULT STDMETHODCALLTYPE webViewDidInvalidate( IWebView* sender);

    virtual HRESULT STDMETHODCALLTYPE createWebViewWithRequest( IWebView* sender,  IWebURLRequest* request,  IPropertyBag* windowFeatures,  IWebView** newWebView) { return E_NOTIMPL; }

    virtual HRESULT STDMETHODCALLTYPE drawBackground( IWebView* sender,  OLE_HANDLE hdc,  const RECT* dirtyRect) { return E_NOTIMPL; }

    virtual HRESULT STDMETHODCALLTYPE decidePolicyForGeolocationRequest( IWebView* sender,  IWebFrame* frame,  IWebSecurityOrigin* origin,  IWebGeolocationPolicyListener* listener) { return E_NOTIMPL; }

private:
    int m_refCount;
};

#endif
